<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Aug 29, 2013 9:22:04 AM
 * @version $Rev$
 */

/**
 * Writer class
 * Used to insert WP posts and pages in the database
 */
abstract class Entity_Writer {
	
	protected $type = false;
	protected $dataDictionary = array();
	protected $postType = 'post';
	protected $type_suffix =  '';

	/**
	 *
	 * @var WP_Interfacer
	 */
	protected $interfacer = null;
    protected $customFieldDictionary;


	public function __construct($dataDictionary, $customFieldDictionary = array())
	{
		if(!$this->type)
		{
			throw new Exception('The writer has no type set');
		}
		
		$this->dataDictionary = $dataDictionary;
        $this->customFieldDictionary = $customFieldDictionary;
		$this->interfacer = WP_Interfacer::getInstance();
	}
	
	/**
	 * used to provide the content of the post or page
	 */
	abstract function getContent();
	
	/**
	 * used to provide the title of the post or page
	 */
	abstract function getTitle();

	/**
	 * used to provide a correspondentID, 
	 * like associated publication or candidate ID
	 */
	abstract function getCorrespondentID();
	
	/**
	 * used to provide a publication date.
	 */
	abstract function getPublishDate();

    /**
     * used to provide the slug if appropriate or false if not desired to change
     */
    abstract function getSlug();
    
	/**
	 * constructs the type string associated as meta property with this post
	 * @return string
	 */
	protected function getType()
	{
		return WP_Constants::META_KEY_PREFIX . $this->type . $this->type_suffix;
	}
	
	public function setTypeSuffix($suffix)
	{
		$this->type_suffix = '_' . $suffix;
	}
	
    /**
     * writes the post in the wordpress database
     * 
     * @return boolean
     */
    public function write() {
        /**
         * return false if it can not find the carerix_user
         */
        if(isset($this->dataDictionary['ownerName']) 
                && strlen(trim($this->dataDictionary['ownerName'])) > 0) {
            $nicename = $this->dataDictionary['ownerName'];
        } else {
            $nicename = WP_Constants::CARERIX_USER_NICENAME;
        }

        $nicename = trim(sanitize_user($nicename, true));

        $user_id = $this->interfacer->user_id_by_login($nicename);
        if($user_id === FALSE)
        {
            // the user doesn't exist, so create it
            $user_id = $this->interfacer->user_insert($nicename, Environment_Installation::randomPassword());

            if ($user_id === FALSE) {
                return FALSE;
            }
        }

        /**
         * if it can not get a publish date, it will use the current date
         */
        $publishDate = $this->getPublishDate();
        if(!$publishDate)
        {
            $publishDate = date('Y-m-d');
        } 
        else 
        {
            $publishDate = date('Y-m-d', strtotime((string)$publishDate));
        }

        $data = array(
            'post_title' => $this->getTitle(),
            'post_type' => $this->postType,
            'post_content' => $this->getContent(),
            'post_date' => $publishDate,
            'post_author' => $user_id,
        );

        $slug = $this->getSlug();
        if ($slug) {
            $data['post_name'] = $slug;
        }

        /**
         * check if a post with this corresponding id exists in the database
         * if it exists, update it
         * otherwise, insert it
         */
        $posts = $this->interfacer->entities_get_where_property($this->getType(), $this->getCorrespondentID());
        $id = false;
        $post_count = count($posts);
        if ($post_count > 0) {
            for ($i = 0; $i < $post_count; $i++) {
                if (isset($posts[$i]->post_id) && $posts[$i]->post_id !== NULL) {
                    $id = $posts[$i]->post_id;

                    $d = $this->interfacer->entity_get($id);
                    // the post doesn't actually exist, so create a new one
                    // [todo] clean wpml
                    if ($d == null) {
                        // clean metadata for nonexistent post
                        $this->interfacer->entities_delete_where_property($id);
                        $id = false;
                    } else {
                        break;
                    }
                }
            }
        }
        $posts = null;
        
        if($id)
            {
            $d = $this->interfacer->entity_get($id);
            $data['post_status'] = @$d['post_status'];
            if($data['post_status'] == 'trash')
            {
                return $id;
            }
            $this->interfacer->entity_update($id, $data);
			wp_cache_delete( $id, 'posts' );
        }
        else
        {
            $data['post_status'] = 'publish';
            $id = $this->interfacer->entity_insert($data);
        }

        /**
         * set the correspondeID for this post.
         */
        $this->interfacer->entity_set_property($id, $this->getType(), $this->getCorrespondentID());

        $this->interfacer->entity_set_property($id, 'managed_by', WP_Constants::META_KEY_PREFIX);

        foreach($this->customFieldDictionary as $field => $value)
        {
            $this->interfacer->entity_set_property($id, $field, $value);
        }

        /**
         * if the writer has a postWrite method, 
         * launch it with the post id as parameter
         */
        if( method_exists($this, 'postWrite') )
        {
            $this->postWrite($id);
        }

		$extra = array();
		if ( isset($this->cxfields) ) { // Defined in Writer\Job.php
			$extra[] = $this->cxfields;
		}

        do_action( 'cxwordpress_post_updated', array($id), array($data), $extra );
        
        /**
         * return the post id parameter
         */
        return $id;
    }
}